@@ -72,32 +72,41 @@ static guint signals [LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (EmpathyIndividualMenu, empathy_individual_menu, GTK_TYPE_MENU);
 
-static GtkWidget * empathy_individual_chat_menu_item_new (
-    EmpathyIndividualMenu *self, FolksIndividual *individual);
-static GtkWidget * empathy_individual_sms_menu_item_new (
-    EmpathyIndividualMenu *self, FolksIndividual *individual);
-static GtkWidget * empathy_individual_log_menu_item_new  (
+static GtkWidget * chat_menu_item_new_individual (EmpathyIndividualMenu *self,
     FolksIndividual *individual);
-static GtkWidget * empathy_individual_info_menu_item_new (
-    FolksIndividual *individual);
-static GtkWidget * empathy_individual_edit_menu_item_new (
-    FolksIndividual *individual);
-static GtkWidget * empathy_individual_invite_menu_item_new (
-    FolksIndividual *individual,
+static GtkWidget * chat_menu_item_new_contact (EmpathyIndividualMenu *self,
     EmpathyContact *contact);
-static GtkWidget * empathy_individual_file_transfer_menu_item_new (
-    EmpathyIndividualMenu *self, FolksIndividual *individual);
-static GtkWidget * empathy_individual_share_my_desktop_menu_item_new (
-    EmpathyIndividualMenu *self, FolksIndividual *individual);
-static GtkWidget * empathy_individual_favourite_menu_item_new (
+static GtkWidget * sms_menu_item_new_individual (EmpathyIndividualMenu *self,
     FolksIndividual *individual);
-static GtkWidget * empathy_individual_add_menu_item_new (
+static GtkWidget * sms_menu_item_new_contact (EmpathyIndividualMenu *self,
+    EmpathyContact *contact);
+static GtkWidget * audio_call_menu_item_new_contact (
     EmpathyIndividualMenu *self,
+    EmpathyContact *contact);
+static GtkWidget * video_call_menu_item_new_contact (
+    EmpathyIndividualMenu *self,
+    EmpathyContact *contact);
+static GtkWidget * log_menu_item_new_individual  (FolksIndividual *individual);
+static GtkWidget * log_menu_item_new_contact (EmpathyContact *contact);
+static GtkWidget * info_menu_item_new_individual (FolksIndividual *individual);
+static GtkWidget * edit_menu_item_new_individual (FolksIndividual *individual);
+static GtkWidget * invite_menu_item_new (FolksIndividual *individual,
+    EmpathyContact *contact);
+static GtkWidget * file_transfer_menu_item_new_individual (EmpathyIndividualMenu *self,
     FolksIndividual *individual);
-static GtkWidget * empathy_individiual_block_menu_item_new (
+static GtkWidget * file_transfer_menu_item_new_contact (
+    EmpathyIndividualMenu *self,
+    EmpathyContact *contact);
+static GtkWidget * share_my_desktop_menu_item_new_individual (EmpathyIndividualMenu *self,
     FolksIndividual *individual);
-static GtkWidget * empathy_individiual_remove_menu_item_new (
-    EmpathyIndividualMenu *self);
+static GtkWidget * share_my_desktop_menu_item_new_contact (
+    EmpathyIndividualMenu *self,
+    EmpathyContact *contact);
+static GtkWidget * favourite_menu_item_new_individual (FolksIndividual *individual);
+static GtkWidget * add_menu_item_new_individual (EmpathyIndividualMenu *self,
+    FolksIndividual *individual);
+static GtkWidget * block_menu_item_new_individual (FolksIndividual *individual);
+static GtkWidget * remove_menu_item_new_individual (EmpathyIndividualMenu *self);
 
 static void
 individual_menu_add_personas (EmpathyIndividualMenu *self,
@@ -153,8 +162,6 @@ individual_menu_add_personas (EmpathyIndividualMenu *self,
       FolksPersonaStore *store;
       const gchar *account;
       GtkWidget *action;
-      /* Individual containing only persona */
-      FolksIndividual *single_individual;
 
       if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (persona)))
         goto while_finish;
@@ -164,13 +171,6 @@ individual_menu_add_personas (EmpathyIndividualMenu *self,
         goto while_finish;
 
       contact = empathy_contact_dup_from_tp_contact (tp_contact);
-      single_individual = empathy_ensure_individual_from_tp_contact (
-          tp_contact);
-
-      /* Pretty hacky. Creating single_individual had a side effect to change
-       * persona.individual from individual to single_individual which is not
-       * what we want so we set it back. See bgo#684971 for details. */
-      g_object_set (persona, "individual", individual, NULL);
 
       store = folks_persona_get_store (FOLKS_PERSONA (persona));
       account = folks_persona_store_get_display_name (store);
@@ -194,8 +194,7 @@ individual_menu_add_personas (EmpathyIndividualMenu *self,
       /* Chat */
       if (features & EMPATHY_INDIVIDUAL_FEATURE_CHAT)
         {
-          action = empathy_individual_chat_menu_item_new (self,
-              single_individual);
+          action = chat_menu_item_new_contact (self, contact);
           gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
           gtk_widget_show (action);
         }
@@ -203,8 +202,7 @@ individual_menu_add_personas (EmpathyIndividualMenu *self,
       /* SMS */
       if (features & EMPATHY_INDIVIDUAL_FEATURE_SMS)
         {
-          action = empathy_individual_sms_menu_item_new (self,
-              single_individual);
+          action = sms_menu_item_new_contact (self, contact);
           gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
           gtk_widget_show (action);
         }
@@ -212,14 +210,12 @@ individual_menu_add_personas (EmpathyIndividualMenu *self,
       if (features & EMPATHY_INDIVIDUAL_FEATURE_CALL)
         {
           /* Audio Call */
-          action = empathy_individual_audio_call_menu_item_new (
-              self, single_individual);
+          action = audio_call_menu_item_new_contact (self, contact);
           gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
           gtk_widget_show (action);
 
           /* Video Call */
-          action = empathy_individual_video_call_menu_item_new (
-              self, single_individual);
+          action = video_call_menu_item_new_contact (self, contact);
           gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
           gtk_widget_show (action);
         }
@@ -227,51 +223,34 @@ individual_menu_add_personas (EmpathyIndividualMenu *self,
       /* Log */
       if (features & EMPATHY_INDIVIDUAL_FEATURE_LOG)
         {
-          action = empathy_individual_log_menu_item_new (single_individual);
+          action = log_menu_item_new_contact (contact);
           gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
           gtk_widget_show (action);
         }
 
       /* Invite */
-      action = empathy_individual_invite_menu_item_new (NULL, contact);
+      action = invite_menu_item_new (NULL, contact);
       gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
       gtk_widget_show (action);
 
       /* File transfer */
       if (features & EMPATHY_INDIVIDUAL_FEATURE_FILE_TRANSFER)
         {
-          action = empathy_individual_file_transfer_menu_item_new (
-              self, single_individual);
+          action = file_transfer_menu_item_new_contact (self, contact);
           gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
           gtk_widget_show (action);
         }
 
       /* Share my desktop */
-      action = empathy_individual_share_my_desktop_menu_item_new (
-          self, single_individual);
+      action = share_my_desktop_menu_item_new_contact (self, contact);
       gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
       gtk_widget_show (action);
 
-      /* Block */
-      if (features & EMPATHY_INDIVIDUAL_FEATURE_BLOCK &&
-          (item = empathy_individiual_block_menu_item_new (single_individual))
-          != NULL) {
-        GtkWidget *sep;
-
-        sep = gtk_separator_menu_item_new ();
-        gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), sep);
-        gtk_widget_show (sep);
-
-        gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), item);
-        gtk_widget_show (item);
-      }
-
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), contact_item);
       gtk_widget_show (contact_item);
 
       g_free (label);
       g_object_unref (contact);
-      g_object_unref (single_individual);
 
 while_finish:
       g_clear_object (&persona);
@@ -681,7 +660,7 @@ contact_blocked_changed_cb (TpContact *contact,
 }
 
 static GtkWidget *
-empathy_individiual_block_menu_item_new (FolksIndividual *individual)
+block_menu_item_new_individual (FolksIndividual *individual)
 {
   GtkWidget *item;
   GList *contacts, *l;
@@ -921,7 +900,7 @@ remove_activate_cb (GtkMenuItem *menuitem,
 }
 
 static GtkWidget *
-empathy_individiual_remove_menu_item_new (EmpathyIndividualMenu *self)
+remove_menu_item_new_individual (EmpathyIndividualMenu *self)
 {
   GeeSet *personas;
   GeeIterator *iter;
@@ -981,7 +960,7 @@ constructed (GObject *object)
   /* Add contact */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_ADD_CONTACT)
     {
-      item = empathy_individual_add_menu_item_new (self, individual);
+      item = add_menu_item_new_individual (self, individual);
       if (item != NULL)
         {
           gtk_menu_shell_append (GTK_MENU_SHELL (shell), item);
@@ -992,7 +971,7 @@ constructed (GObject *object)
   /* Chat */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_CHAT)
     {
-      item = empathy_individual_chat_menu_item_new (self, individual);
+      item = chat_menu_item_new_individual (self, individual);
       if (item != NULL)
         {
           gtk_menu_shell_append (shell, item);
@@ -1003,7 +982,7 @@ constructed (GObject *object)
   /* SMS */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_SMS)
     {
-      item = empathy_individual_sms_menu_item_new (self, individual);
+      item = sms_menu_item_new_individual (self, individual);
       if (item != NULL)
         {
           gtk_menu_shell_append (shell, item);
@@ -1014,12 +993,14 @@ constructed (GObject *object)
   if (features & EMPATHY_INDIVIDUAL_FEATURE_CALL)
     {
       /* Audio Call */
-      item = empathy_individual_audio_call_menu_item_new (self, individual);
+      item = empathy_individual_audio_call_menu_item_new_individual (self,
+          individual);
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
 
       /* Video Call */
-      item = empathy_individual_video_call_menu_item_new (self, individual);
+      item = empathy_individual_video_call_menu_item_new_individual (self,
+          individual);
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
     }
@@ -1028,14 +1009,14 @@ constructed (GObject *object)
     add_phone_numbers (self);
 
   /* Invite */
-  item = empathy_individual_invite_menu_item_new (individual, NULL);
+  item = invite_menu_item_new (individual, NULL);
   gtk_menu_shell_append (shell, item);
   gtk_widget_show (item);
 
   /* File transfer */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_FILE_TRANSFER)
     {
-      item = empathy_individual_file_transfer_menu_item_new (self, individual);
+      item = file_transfer_menu_item_new_individual (self, individual);
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
     }
@@ -1043,7 +1024,7 @@ constructed (GObject *object)
   /* Share my desktop */
   /* FIXME we should add the "Share my desktop" menu item if Vino is
   a registered handler in MC5 */
-  item = empathy_individual_share_my_desktop_menu_item_new (self, individual);
+  item = share_my_desktop_menu_item_new_individual (self, individual);
   gtk_menu_shell_append (shell, item);
   gtk_widget_show (item);
 
@@ -1064,7 +1045,7 @@ constructed (GObject *object)
   /* Edit */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_EDIT)
     {
-      item = empathy_individual_edit_menu_item_new (individual);
+      item = edit_menu_item_new_individual (individual);
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
     }
@@ -1072,7 +1053,7 @@ constructed (GObject *object)
   /* Log */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_LOG)
     {
-      item = empathy_individual_log_menu_item_new (individual);
+      item = log_menu_item_new_individual (individual);
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
     }
@@ -1080,7 +1061,7 @@ constructed (GObject *object)
   /* Info */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_INFO)
     {
-      item = empathy_individual_info_menu_item_new (individual);
+      item = info_menu_item_new_individual (individual);
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
     }
@@ -1088,14 +1069,14 @@ constructed (GObject *object)
   /* Favorite checkbox */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_FAVOURITE)
     {
-      item = empathy_individual_favourite_menu_item_new (individual);
+      item = favourite_menu_item_new_individual (individual);
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
     }
 
   /* Separator & Block */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_BLOCK &&
-      (item = empathy_individiual_block_menu_item_new (individual)) != NULL) {
+      (item = block_menu_item_new_individual (individual)) != NULL) {
     GtkWidget *sep;
 
     sep = gtk_separator_menu_item_new ();
@@ -1108,7 +1089,7 @@ constructed (GObject *object)
 
   /* Separator & Remove */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_REMOVE &&
-      (item = empathy_individiual_remove_menu_item_new (self)) != NULL) {
+      (item = remove_menu_item_new_individual (self)) != NULL) {
     GtkWidget *sep;
 
     sep = gtk_separator_menu_item_new ();
@@ -1362,15 +1343,11 @@ empathy_individual_chat_menu_item_activated (GtkMenuItem *item,
 }
 
 static GtkWidget *
-empathy_individual_chat_menu_item_new (EmpathyIndividualMenu *self,
-    FolksIndividual *individual)
+chat_menu_item_new (EmpathyIndividualMenu *self)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) &&
-      empathy_folks_individual_contains_contact (individual), NULL);
-
   item = gtk_image_menu_item_new_with_mnemonic (_("_Chat"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_MESSAGE,
       GTK_ICON_SIZE_MENU);
@@ -1378,6 +1355,21 @@ empathy_individual_chat_menu_item_new (EmpathyIndividualMenu *self,
   gtk_widget_show (image);
 
   g_object_set_data (G_OBJECT (item), "individual-menu", self);
+
+  return item;
+}
+
+static GtkWidget *
+chat_menu_item_new_individual (EmpathyIndividualMenu *self,
+    FolksIndividual *individual)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) &&
+      empathy_folks_individual_contains_contact (individual), NULL);
+
+  item = chat_menu_item_new (self);
+
   menu_item_set_first_contact (item, individual,
       G_CALLBACK (empathy_individual_chat_menu_item_activated),
       EMPATHY_ACTION_CHAT);
@@ -1385,6 +1377,23 @@ empathy_individual_chat_menu_item_new (EmpathyIndividualMenu *self,
   return item;
 }
 
+static GtkWidget *
+chat_menu_item_new_contact (EmpathyIndividualMenu *self,
+    EmpathyContact *contact)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+  item = chat_menu_item_new (self);
+
+  menu_item_set_contact (item, contact,
+      G_CALLBACK (empathy_individual_chat_menu_item_activated),
+      EMPATHY_ACTION_CHAT);
+
+  return item;
+}
+
 static void
 empathy_individual_sms_menu_item_activated (GtkMenuItem *item,
   EmpathyContact *contact)
@@ -1401,15 +1410,11 @@ empathy_individual_sms_menu_item_activated (GtkMenuItem *item,
 }
 
 static GtkWidget *
-empathy_individual_sms_menu_item_new (EmpathyIndividualMenu *self,
-    FolksIndividual *individual)
+sms_menu_item_new (EmpathyIndividualMenu *self)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) &&
-      empathy_folks_individual_contains_contact (individual), NULL);
-
   item = gtk_image_menu_item_new_with_mnemonic (_("_SMS"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_SMS,
       GTK_ICON_SIZE_MENU);
@@ -1417,6 +1422,21 @@ empathy_individual_sms_menu_item_new (EmpathyIndividualMenu *self,
   gtk_widget_show (image);
 
   g_object_set_data (G_OBJECT (item), "individual-menu", self);
+
+  return item;
+}
+
+static GtkWidget *
+sms_menu_item_new_individual (EmpathyIndividualMenu *self,
+    FolksIndividual *individual)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) &&
+      empathy_folks_individual_contains_contact (individual), NULL);
+
+  item = sms_menu_item_new (self);
+
   menu_item_set_first_contact (item, individual,
       G_CALLBACK (empathy_individual_sms_menu_item_activated),
       EMPATHY_ACTION_SMS);
@@ -1424,6 +1444,24 @@ empathy_individual_sms_menu_item_new (EmpathyIndividualMenu *self,
   return item;
 }
 
+static GtkWidget *
+sms_menu_item_new_contact (EmpathyIndividualMenu *self,
+    EmpathyContact *contact)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+  item = sms_menu_item_new (self);
+
+  menu_item_set_contact (item, contact,
+      G_CALLBACK (empathy_individual_sms_menu_item_activated),
+      EMPATHY_ACTION_SMS);
+
+  return item;
+}
+
+
 static void
 empathy_individual_audio_call_menu_item_activated (GtkMenuItem *item,
   EmpathyContact *contact)
@@ -1437,21 +1475,33 @@ empathy_individual_audio_call_menu_item_activated (GtkMenuItem *item,
   emit_menu_item_activated (item);
 }
 
-GtkWidget *
-empathy_individual_audio_call_menu_item_new (EmpathyIndividualMenu *self,
-    FolksIndividual *individual)
+static GtkWidget *
+audio_call_menu_item_new (EmpathyIndividualMenu *self)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
-
   item = gtk_image_menu_item_new_with_mnemonic (C_("menu item", "_Audio Call"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_VOIP, GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);
 
   g_object_set_data (G_OBJECT (item), "individual-menu", self);
+
+  return item;
+}
+
+GtkWidget *
+empathy_individual_audio_call_menu_item_new_individual (
+    EmpathyIndividualMenu *self,
+    FolksIndividual *individual)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
+
+  item = audio_call_menu_item_new (self);
+
   menu_item_set_first_contact (item, individual,
       G_CALLBACK (empathy_individual_audio_call_menu_item_activated),
       EMPATHY_ACTION_AUDIO_CALL);
@@ -1459,6 +1509,25 @@ empathy_individual_audio_call_menu_item_new (EmpathyIndividualMenu *self,
   return item;
 }
 
+static GtkWidget *
+audio_call_menu_item_new_contact (
+    EmpathyIndividualMenu *self,
+    EmpathyContact *contact)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+  item = audio_call_menu_item_new (self);
+
+  menu_item_set_contact (item, contact,
+      G_CALLBACK (empathy_individual_audio_call_menu_item_activated),
+      EMPATHY_ACTION_AUDIO_CALL);
+
+  return item;
+}
+
+
 static void
 empathy_individual_video_call_menu_item_activated (GtkMenuItem *item,
   EmpathyContact *contact)
@@ -1472,15 +1541,11 @@ empathy_individual_video_call_menu_item_activated (GtkMenuItem *item,
   emit_menu_item_activated (item);
 }
 
-GtkWidget *
-empathy_individual_video_call_menu_item_new (EmpathyIndividualMenu *self,
-    FolksIndividual *individual)
+static GtkWidget *
+video_call_menu_item_new (EmpathyIndividualMenu *self)
 {
   GtkWidget *item;
   GtkWidget *image;
-  TpawCameraMonitor *monitor;
-
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
 
   item = gtk_image_menu_item_new_with_mnemonic (C_("menu item", "_Video Call"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_VIDEO_CALL,
@@ -1489,9 +1554,14 @@ empathy_individual_video_call_menu_item_new (EmpathyIndividualMenu *self,
   gtk_widget_show (image);
 
   g_object_set_data (G_OBJECT (item), "individual-menu", self);
-  menu_item_set_first_contact (item, individual,
-      G_CALLBACK (empathy_individual_video_call_menu_item_activated),
-      EMPATHY_ACTION_VIDEO_CALL);
+
+  return item;
+}
+
+static void
+check_camera_available (GtkWidget *item)
+{
+  TpawCameraMonitor *monitor;
 
   /* Only follow available cameras if the contact can do Video calls */
   if (gtk_widget_get_sensitive (item))
@@ -1502,6 +1572,43 @@ empathy_individual_video_call_menu_item_new (EmpathyIndividualMenu *self,
       g_object_bind_property (monitor, "available", item, "sensitive",
           G_BINDING_SYNC_CREATE);
     }
+}
+
+GtkWidget *
+empathy_individual_video_call_menu_item_new_individual (
+    EmpathyIndividualMenu *self,
+    FolksIndividual *individual)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
+
+  item = video_call_menu_item_new (self);
+
+  menu_item_set_first_contact (item, individual,
+      G_CALLBACK (empathy_individual_video_call_menu_item_activated),
+      EMPATHY_ACTION_VIDEO_CALL);
+
+  check_camera_available (item);
+
+  return item;
+}
+
+GtkWidget *
+video_call_menu_item_new_contact (EmpathyIndividualMenu *self,
+    EmpathyContact *contact)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+  item = video_call_menu_item_new (self);
+
+  menu_item_set_contact (item, contact,
+      G_CALLBACK (empathy_individual_video_call_menu_item_activated),
+      EMPATHY_ACTION_VIDEO_CALL);
+
+  check_camera_available (item);
 
   return item;
 }
@@ -1517,18 +1624,28 @@ empathy_individual_log_menu_item_activated (GtkMenuItem *item,
 }
 
 static GtkWidget *
-empathy_individual_log_menu_item_new (FolksIndividual *individual)
+log_menu_item_new (void)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
-
   item = gtk_image_menu_item_new_with_mnemonic (_("_Previous Conversations"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_LOG, GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);
 
+  return item;
+}
+
+static GtkWidget *
+log_menu_item_new_individual (FolksIndividual *individual)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
+
+  item = log_menu_item_new ();
+
   menu_item_set_first_contact (item, individual,
       G_CALLBACK (empathy_individual_log_menu_item_activated),
       EMPATHY_ACTION_VIEW_LOGS);
@@ -1536,6 +1653,22 @@ empathy_individual_log_menu_item_new (FolksIndividual *individual)
   return item;
 }
 
+static GtkWidget *
+log_menu_item_new_contact (EmpathyContact *contact)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+  item = log_menu_item_new ();
+
+  menu_item_set_contact (item, contact,
+      G_CALLBACK (empathy_individual_log_menu_item_activated),
+      EMPATHY_ACTION_VIEW_LOGS);
+
+  return item;
+}
+
 static void
 empathy_individual_file_transfer_menu_item_activated (GtkMenuItem *item,
     EmpathyContact *contact)
@@ -1548,14 +1681,11 @@ empathy_individual_file_transfer_menu_item_activated (GtkMenuItem *item,
 }
 
 static GtkWidget *
-empathy_individual_file_transfer_menu_item_new (EmpathyIndividualMenu *self,
-    FolksIndividual *individual)
+file_transfer_menu_item_new (EmpathyIndividualMenu *self)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
-
   item = gtk_image_menu_item_new_with_mnemonic (_("Send File"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_DOCUMENT_SEND,
       GTK_ICON_SIZE_MENU);
@@ -1563,6 +1693,20 @@ empathy_individual_file_transfer_menu_item_new (EmpathyIndividualMenu *self,
   gtk_widget_show (image);
 
   g_object_set_data (G_OBJECT (item), "individual-menu", self);
+
+  return item;
+}
+
+static GtkWidget *
+file_transfer_menu_item_new_individual (EmpathyIndividualMenu *self,
+    FolksIndividual *individual)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
+
+  item = file_transfer_menu_item_new (self);
+
   menu_item_set_first_contact (item, individual,
       G_CALLBACK (empathy_individual_file_transfer_menu_item_activated),
       EMPATHY_ACTION_SEND_FILE);
@@ -1570,6 +1714,23 @@ empathy_individual_file_transfer_menu_item_new (EmpathyIndividualMenu *self,
   return item;
 }
 
+static GtkWidget *
+file_transfer_menu_item_new_contact (EmpathyIndividualMenu *self,
+    EmpathyContact *contact)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+  item = file_transfer_menu_item_new (self);
+
+  menu_item_set_contact (item, contact,
+      G_CALLBACK (empathy_individual_file_transfer_menu_item_activated),
+      EMPATHY_ACTION_SEND_FILE);
+
+  return item;
+}
+
 static void
 empathy_individual_share_my_desktop_menu_item_activated (GtkMenuItem *item,
     EmpathyContact *contact)
@@ -1582,20 +1743,31 @@ empathy_individual_share_my_desktop_menu_item_activated (GtkMenuItem *item,
 }
 
 static GtkWidget *
-empathy_individual_share_my_desktop_menu_item_new (EmpathyIndividualMenu *self,
-    FolksIndividual *individual)
+share_my_desktop_menu_item_new (EmpathyIndividualMenu *self)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
-
   item = gtk_image_menu_item_new_with_mnemonic (_("Share My Desktop"));
   image = gtk_image_new_from_icon_name (GTK_STOCK_NETWORK, GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);
 
   g_object_set_data (G_OBJECT (item), "individual-menu", self);
+
+  return item;
+}
+
+static GtkWidget *
+share_my_desktop_menu_item_new_individual (EmpathyIndividualMenu *self,
+    FolksIndividual *individual)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
+
+  item = share_my_desktop_menu_item_new (self);
+
   menu_item_set_first_contact (item, individual,
       G_CALLBACK (empathy_individual_share_my_desktop_menu_item_activated),
       EMPATHY_ACTION_SHARE_MY_DESKTOP);
@@ -1603,6 +1775,23 @@ empathy_individual_share_my_desktop_menu_item_new (EmpathyIndividualMenu *self,
   return item;
 }
 
+static GtkWidget *
+share_my_desktop_menu_item_new_contact (EmpathyIndividualMenu *self,
+    EmpathyContact *contact)
+{
+  GtkWidget *item;
+
+  g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+  item = share_my_desktop_menu_item_new (self);
+
+  menu_item_set_contact (item, contact,
+      G_CALLBACK (empathy_individual_share_my_desktop_menu_item_activated),
+      EMPATHY_ACTION_SHARE_MY_DESKTOP);
+
+  return item;
+}
+
 static void
 favourite_menu_item_toggled_cb (GtkCheckMenuItem *item,
   FolksIndividual *individual)
@@ -1613,7 +1802,7 @@ favourite_menu_item_toggled_cb (GtkCheckMenuItem *item,
 }
 
 static GtkWidget *
-empathy_individual_favourite_menu_item_new (FolksIndividual *individual)
+favourite_menu_item_new_individual (FolksIndividual *individual)
 {
   GtkWidget *item;
 
@@ -1637,7 +1826,7 @@ individual_info_menu_item_activate_cb (GtkMenuItem *item,
 }
 
 static GtkWidget *
-empathy_individual_info_menu_item_new (FolksIndividual *individual)
+info_menu_item_new_individual (FolksIndividual *individual)
 {
   GtkWidget *item;
   GtkWidget *image;
@@ -1666,7 +1855,7 @@ individual_edit_menu_item_activate_cb (FolksIndividual *individual)
 }
 
 static GtkWidget *
-empathy_individual_edit_menu_item_new (FolksIndividual *individual)
+edit_menu_item_new_individual (FolksIndividual *individual)
 {
   EmpathyIndividualManager *manager;
   GtkWidget *item;
@@ -1835,7 +2024,7 @@ create_room_sub_menu (FolksIndividual *individual,
 }
 
 static GtkWidget *
-empathy_individual_invite_menu_item_new (FolksIndividual *individual,
+invite_menu_item_new (FolksIndividual *individual,
     EmpathyContact *contact)
 {
   GtkWidget *item;
@@ -1977,7 +2166,7 @@ add_menu_item_activated (GtkMenuItem *item,
 }
 
 static GtkWidget *
-empathy_individual_add_menu_item_new (EmpathyIndividualMenu *self,
+add_menu_item_new_individual (EmpathyIndividualMenu *self,
     FolksIndividual *individual)
 {
   EmpathyIndividualMenuPriv *priv = GET_PRIV (self);
