@@ -22,10 +22,9 @@
 #ifndef __EMPATHY_EVENT_MANAGER_H__
 #define __EMPATHY_EVENT_MANAGER_H__
 
-#include <glib.h>
 #include <glib-object.h>
 
-#include <libempathy/empathy-contact.h>
+#include "empathy-contact.h"
 
 G_BEGIN_DECLS
 
